data {
    int N;
    int M;
    int K;
    array[N, M] int<lower = 1, upper = K> X;
    vector[K-2] a;
}
transformed data {
    real C1;
    real CK_1;
    //
    //     These values specify the origin and unit of the scale
    //     cf. Okamoto (2013). Behaviormetrika, 40, 149-168.
    //
    C1 = -1;
    CK_1 = 1;
}
parameters {
    array[M] real mu;
    array[M] real<lower = 0.0> Lmd;
    array[M] real<lower = 0.0> psy;
    simplex[K-2] Ck; 
    array[N] real F;
}
transformed parameters {
    array[N, M] vector[K] p;
    array[N, M] vector[K-1] cumP;
    array[K-1] real C;

    C[1] = C1;
    C[K-1] = CK_1;
    for (k in 1:(K-3)) {
        C[k+1] = C[k] + Ck[k] * 2.0;
    }
    for (i in 1:N)
        for (j in 1:M){
            cumP[i][j][K-1] = normal_cdf((C[K-1] - (mu[j] + Lmd[j] * F[i])) / psy[j] | 0.0, 1.0);
            cumP[i][j][1] = normal_cdf((C[1] - (mu[j] + Lmd[j] * F[i])) / psy[j] | 0.0, 1.0);
            for (k in 1:(K-3)) {
                cumP[i][j][k+1] = normal_cdf((C[k+1] - (mu[j] + Lmd[j] * F[i])) / psy[j] | 0.0, 1.0);
            }
            p[i][j][1] = cumP[i][j][1];
            for (k in 1:(K-2)) {
                p[i][j][k+1] = cumP[i][j][k+1] - cumP[i][j][k]; 
            }
            p[i][j][K] = 1.0 - cumP[i][j][K-1];
        }
    
}
model {
    for (j in 1:M) {
        mu[j] ~ normal(0.0, 10.0);
        psy[j] ~ exponential(0.1);
        Lmd[j] ~ exponential(0.1);
    }
    Ck ~ dirichlet(a);
    F ~ normal(0.0, 1.0);
    for (i in 1:N) {
        for (j in 1:M) {
            X[i][j] ~ categorical(p[i][j]);
        }
    }
}
generated quantities {
    real sdF;
    real mnF;
    real sumF;
    real ssF;
    array[N] real cF;
    array[M] real cLmd;
    sumF = 0.0;
    ssF = 0.0;
    for (i in 1:N) {
        sumF += F[i];
        ssF += pow(F[i], 2);
    }
    mnF = sumF / N;
    sdF = pow((ssF - N * pow(mnF, 2)) / (N - 1), 0.5);
    for (i in 1:N) {
        cF[i] = (F[i] - mnF) / sdF;
    }
    for (j in 1:M) {
        cLmd[j] = Lmd[j] * sdF;
    }
}
